/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __ICONS_H
#define __ICONS_H

#include <windows.h>
#include <windowsx.h>
#include <shlwapi.h>
#include "contextmenu.h"
#include "..\lsapi\lsapi.h"
#include "..\lsapi\lswinbase.h"

struct DESKTOPICON
{
	int x;
	int y;
	RECT iconRect;
	RECT textRect;
	HICON icon;
	char title[MAX_PATH];
	char filename[MAX_PATH];
	DWORD attrib;
	DESKTOPICON *next;
};

enum DESKTOPDIRECTION
{
	DD_RIGHT,
	DD_DOWN,
	DD_LEFT,
	DD_UP
};

class Icons
{
public:
	Icons(const char *name, HWND parentWnd);
	~Icons();

	void show();
	void hide();
	void toggle();

	void paintIcons(HDC hDC);
	void update();
	BOOL mouseEvent(UINT uMsg, WPARAM wParam, LPARAM lParam);
	
	Icons* next;
	char* name;
	HANDLE iconChangeHandle;

private:
	HWND _parentWnd;
	BOOL _visible;

	DESKTOPICON *_iconList;
	BOOL _iconsLoaded;

	int _iconX;
	int _iconY;
	int _iconSize;
	int _iconSpacingX;
	int _iconSpacingY;
	char _iconFolder[MAX_PATH];

	BOOL _iconFoldersFirst;
	BOOL _iconHideExtension;
	BOOL _iconShowHidden;

	DESKTOPDIRECTION _iconDirection;
	DESKTOPDIRECTION _iconWrapDirection;
	int _iconWrapCount;

	int _iconTextSpacing;
	int _iconTextHeight;
	int _iconTextWidth;
	HFONT _iconTextFont;
	COLORREF _iconTextColor;
	HBRUSH _iconTextBackground;

	void _loadIcons();
	void _deleteIcons();

	DESKTOPICON *_getIconAtPt(int x, int y);
	DESKTOPICON *_selectedIcon;

	HICON _getIconFromPathEx(LPCSTR pszPath, int cxDesired, int cyDesired);
	int _parseIconLocation(LPSTR szIconFile);

	void _getString(const char* key, char* value, const char* defaultValue, int maxLen);
	BOOL _getBool(const char* key, BOOL defaultValue);
	int _getInt(const char* key, int defaultValue);
	COLORREF _getColor(const char* key, int defaultValue);

};

#endif//__ICONS_H
