/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "main.h"

int initModuleEx(HWND hParentWnd, HINSTANCE hInstance, LPCSTR szPath)
{
	desktopWnd = FindWindow("DesktopBackgroundClass", NULL);
	if(!desktopWnd) {
		LSLog(LOG_ERROR, "IconDesk", "Could not find desktop window");
	}

	oldWndProc = (WNDPROC)GetWindowLong(desktopWnd, GWL_WNDPROC);
	SetWindowLong(desktopWnd, GWL_WNDPROC, (LONG)wndProc);
	oldClassStyle = GetClassLong(desktopWnd, GCL_STYLE);
	SetClassLong(desktopWnd, GCL_STYLE, GetClassLong(desktopWnd, GCL_STYLE) | CS_DBLCLKS);

	AddBangCommand("!IconDeskShow", bangShow);
	AddBangCommand("!IconDeskHide", bangHide);
	AddBangCommand("!IconDeskToggle", bangToggle);
	AddBangCommand("!IconDeskUpdate", bangUpdate);

	timerID = GetTickCount();
	SetTimer(desktopWnd, timerID, GetRCInt("IconDeskUpdateInterval", 500), NULL);

	loadGroups();

	InvalidateRect(desktopWnd, NULL, TRUE);

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	SetWindowLong(desktopWnd, GWL_WNDPROC, (LONG)oldWndProc);
	SetClassLong(desktopWnd, GCL_STYLE, oldClassStyle);

	RemoveBangCommand("!IconDeskShow");
	RemoveBangCommand("!IconDeskHide");
	RemoveBangCommand("!IconDeskToggle");
	RemoveBangCommand("!IconDeskUpdate");

	KillTimer(desktopWnd, timerID);

	deleteGroups();

	InvalidateRect(desktopWnd, NULL, TRUE);
}

LRESULT wndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		case WM_MBUTTONUP:
		case WM_LBUTTONDBLCLK:
		case WM_RBUTTONDBLCLK:
		case WM_MBUTTONDBLCLK:
			{
				BOOL hit = FALSE;
				Icons* loop = groupList;
				while(loop) {
					if(loop->mouseEvent(uMsg, wParam, lParam)) {
						hit = TRUE;
					}
					loop = loop->next;
				}
				if(hit) {
					return 0;
				}
			}
			break;

		case WM_PAINT:
			{
				PAINTSTRUCT	ps;
				HDC			hDC;

				if(!wParam) {
					hDC = BeginPaint(hWnd, &ps);
				}
				else {
					hDC = (HDC)wParam;
				}

				Icons* loop = groupList;
				while(loop) {
					loop->paintIcons(hDC);
					loop = loop->next;
				}

				if(!wParam) {
					EndPaint(hWnd, &ps);
				}
			}

		case WM_TIMER:
			if(wParam == timerID) {
				Icons* loop = groupList;
				while(loop) {
					if(WaitForSingleObject(loop->iconChangeHandle, 0) == WAIT_OBJECT_0) {
						loop->update();
						InvalidateRect(desktopWnd, NULL, TRUE);
						FindNextChangeNotification(loop->iconChangeHandle);
					}
					loop = loop->next;
				}
				return 0;
			}
			break;
	}
	return CallWindowProc(oldWndProc, hWnd, uMsg, wParam, lParam);
}

void bangShow(HWND caller, LPCSTR args)
{
	Icons* group = getGroup(args);
	if(group) {
		group->show();
	}
}

void bangHide(HWND caller, LPCSTR args)
{
	Icons* group = getGroup(args);
	if(group) {
		group->hide();
	}
}

void bangToggle(HWND caller, LPCSTR args)
{
	Icons* group = getGroup(args);
	if(group) {
		group->toggle();
	}
}

void bangUpdate(HWND caller, LPCSTR args)
{
	Icons* group = getGroup(args);
	if(group) {
		group->update();
	}
}

void loadGroups()
{
	char buffer[1024] = "";
	char* group;

	GetRCString("IconDeskGroups", buffer, "", 1024);
	group = strtok(buffer, " ,\t");
	while(group) {
		addGroup(group);
		group = strtok(NULL, " ,\t");
	}
}

void addGroup(const char* name)
{
	Icons* group = new Icons(name, desktopWnd);
	group->next = groupList;
	groupList = group;
}

void deleteGroups()
{
	Icons* loop = groupList;
	Icons* del;
	while(loop) {
		del = loop;
		loop = loop->next;
		delete del;
	}
}

Icons* getGroup(const char* name)
{
	Icons* loop = groupList;
	while(loop) {
		if(!strcmpi(loop->name, name)) {
			return loop;
		}
		loop = loop->next;
	}
	return NULL;
}
